	program eab1
c
c a routine to compute the atmosphere parameters of the
c cmd sub-groups for 47 Tuc (NGC 104), from observed photometry
c
        real v(40),vi(40),teff(40),logg(40),mic(40),frac(40),r(40)
	real feh
	integer nbox,nstar(40)
	data logg/40*2.5/

c
c here are the parameters specific to 47 Tuc
c
	feh  =  -0.70
	dist =  13.50
	ebv  =   0.03                                  ! Gratton et al. (2003)/Harris96/Schelegel98

	call read_kurucz_data(feh)
	call read_gc_boxes(v,vi,frac,nstar,nbox)
	call de_redden(v,vi,ebv,nbox)
	call compute_params(v,vi,feh,teff,logg,mic,r,dist,nbox)
	call write_params(v,vi,teff,logg,mic,r,frac,nstar,nbox)

	stop
	end


	subroutine read_gc_boxes(v,vi,frac,nstar,nbox)
c
c a simple routine to read the cmd information
c
	character*40 line
	real v(40),vi(40),frac(40)
	integer nbox,nstar(40)

	open(unit=8,file='fractions.dat',status='old')
	
	read(8,'(a40)')line
	read(8,'(a40)')line

	do i =1,40
	   read(8,*,end=900)v(i),vi(i),frac(i),nstar(i)
	enddo
 900	continue
	nbox = i-1
	close(unit=8)

	return
	end



	subroutine de_redden(v,vi,ebv,nbox)

	real v(40),vi(40),evi,av
c
c correct V-I for reddning and V for extinction using the
c reddening law of Winkler, 1997, MNRAS, 287, 481.
c
	av = 3.136 * ebv
        evi = 1.281 * ebv
	do i = 1,nbox
	   vi(i) = vi(i) - evi
	   v(i) = v(i) - av
	enddo

	return
	end




        subroutine write_params(v,vi,teff,logg,mic,r,frac,nstar,nbox)

	real v(40),vi(40),teff(40),logg(40),mic(40),r(40),frac(40)
	integer nstar(40)

	write(6,100)
 100	format('    Vavg   (V-I)  Teff   logg   micro  radius',
     %         '   fraction  Nstar',/)
        write(6,200)(v(i),vi(i),teff(i),logg(i),mic(i),r(i),frac(i),
     %               nstar(i),i=1,nbox)
 200	format(2f8.3,f7.0,f7.3,f7.2,f8.2,f10.5,i8)

	return
	end




	subroutine compute_params(v,vi,feh,teff,logg,mic,r,dist,nbox)

	real v(40),vi(40),teff(40),logg(40),mic(40),r(40)
	real l(40),mass,dist,mv
	integer nbox

c
c note that I assume a turnoff mass of 0.8Msun
c
	mass = 0.80

	do i = 1,nbox
	   mv = v(i) - dist
	   logg(i) = 2.5

c
c iterate to converge on BC, logg, and Teff
c
           do j = 1,10

           call get_teff(vi(i),mv,feh,logg(i),teff(i))
           call get_luminosity(v(i),vi(i),teff(i),logg(i),feh,dist,l(i))
           oldlogg = logg(i)
           call compute_logg(l(i),mass,teff(i),logg(i))

           if (abs(logg(i)-oldlogg).lt.0.05) goto 900
           enddo
 900	   continue

           call compute_stellar_radii(l(i),teff(i),r(i))
	   call compute_micro(teff(i),logg(i),mic(i))
	enddo

	return
	end





	subroutine get_teff(vi,mv,feh,logg,teff)

	real vi,vic,vij,mv,feh,logg,teff
c
c Assume (V-I) colors, with calibration from Alonso (1999) for
c giants, and Alonso (1996) for dwarf stars.
c

c First convert from Cousin's to Johnson

        vic = vi

        if (vic .lt. 0.0) then
           vij = vic/0.713
        elseif (vic.ge.0.0 .and. vic.le.1.556) then
           vij = vic/0.778
        elseif (vic.gt.1.556 .and. vic.le.2.375) then
           vij = (vic + 0.13)/0.835
        else
           write(6,'(26h Error: V-I out of bounds )')
           stop
        endif



c giants
        th = 0.5379 + 0.3981*vij + 4.432e-2 * vij**2 - 2.693e-2*vij**3
        teffrgb = 5040./th

c dwarfs

	theta = 0.424 + 0.610*vij - 0.096*vij**2
	teffdwf = 5040./theta

c subgiants
c
c NOTE: the mv cutoffs are based on the NGC 6397 cmd; logg may be better?
c

	if (mv.gt.3.75) then
	   teff=teffdwf
	elseif (mv.lt.3.13) then
	   teff = teffrgb
	elseif (mv.le.3.75 .and. mv.ge.3.13) then
	   f = (mv-3.13)/0.62
	   teff = teffrgb + f*(teffdwf-teffrgb)
	endif

	return
	end




	subroutine get_luminosity(v,vi,teff,logg,feh,dist,l)

	real v,vi,teff,dist,l,mv,mbol_sun,logg,bc,mbol

	mbol_sun =  4.74
	mbol = v - dist + bc(teff,logg,feh)

	l = 10.0**(-0.4*(mbol-mbol_sun))

	return
	end





	real function bc(teff,logg,feh)

	real logg,teff,teff_lo,teff_hi
	real bc_entry,feh
        real bc_lo1,bc_hi1,bc_lo2,bc_hi2

	teff_lo = aint(teff/250.0) * 250.0
	teff_hi = aint(teff/250.0) * 250.0 + 250.0

	fm05 = -0.5
	fm10 = -1.0

        bc_lo1 = bc_entry(teff_lo,logg,fm05)
        bc_hi1 = bc_entry(teff_hi,logg,fm05)

        bc_lo2 = bc_entry(teff_lo,logg,fm10)
        bc_hi2 = bc_entry(teff_hi,logg,fm10)

        bc1 = bc_lo1 + (teff-teff_lo)*(bc_hi1-bc_lo1)/250.0
        bc2 = bc_lo2 + (teff-teff_lo)*(bc_hi2-bc_lo2)/250.0

        bc = bc1 + (bc2-bc1)*(feh-fm05)/(fm10-fm05)

c DEBUG
	write(6,*)'teff,logg,bc'
	write(6,*)teff,logg,bc
c
	return
	end






	real function bc_entry(teff,logg,feh)
c
c here use the Kurucz tables of computed BC
c
c in this version I average the -0.5 and -1.0 [Fe/H] to get tables 
c at -0.75, appropriate for 47 Tuc
c
        common /akurucz/tkur1,tkur2,gkur1,gkur2,bckur1,bckur2
	real tkur1(450),gkur1(450),bckur1(450)
	real tkur2(450),gkur2(450),bckur2(450)

        common /ikurucz/nkur1,nkur2
        integer nkur1,nkur2

	real logg,teff,feh

	bc_entry = -99.99
c
c first the -0.5 [Fe/H] table
c
	if (feh.eq.-0.5) then

	do i = 1,nkur1
	   if (teff-tkur1(i) .le. 50.) then

	      if (logg.lt.gkur1(i)) then    ! if lower than first logg
	         bc_entry = bckur1(i)
	         return
	      endif
	     
              do j = 1,10
	         if (tkur1(i+j).eq.tkur1(i)) then
	            if (gkur1(i+j-1).le.logg .and.
     %                              gkur1(i+j).ge.logg) then
                       f = (logg-gkur1(i+j-1))/(gkur1(i+j)-gkur1(i+j-1))
                       bc_entry = bckur1(i+j-1) + 
     %                           f*(bckur1(i+j)-bckur1(i+j-1))
                       return
                    endif
                 else
                    bc_entry = bckur1(i+j-1)    ! if higher than last logg
                    return
                 endif
              enddo

           endif
        enddo

c
c then the -1.0 [Fe/H] table
c
	elseif (feh.eq.-1.0) then

	do i = 1,nkur2
	   if (teff-tkur2(i) .le. 50.) then

	      if (logg.lt.gkur2(i)) then    ! if lower than first logg
	         bc_entry = bckur2(i)
	         return
	      endif
	     
              do j = 1,10
	         if (tkur2(i+j).eq.tkur2(i)) then
	            if (gkur2(i+j-1).le.logg .and.
     %                              gkur2(i+j).ge.logg) then
                       f = (logg-gkur2(i+j-1))/(gkur2(i+j)-gkur2(i+j-1))
                       bc_entry = bckur2(i+j-1) + 
     %                           f*(bckur2(i+j)-bckur2(i+j-1))
                       return
                    endif
                 else
                    bc_entry = bckur2(i+j-1)    ! if higher than last logg
                    return
                 endif
              enddo

           endif
        enddo

	endif




	write(6,'(a)')'Error: BC not found'
	stop
	end






        subroutine compute_logg(l,mass,teff,logg)

	real l,mass,teff,logg,logg_sun

	tsun = 5777.
	logg_sun = 4.43775
        logg = logg_sun+alog10(mass) - alog10(l) + 4.*alog10(teff/tsun)

	return
	end



	subroutine compute_micro(teff,logg,mic)
c
c a very rough estimation of the trend of microturbulence with logg
c
c FEEL FREE TO IMPROVE THIS ROUTINE
c
c made-up, with some input from MPSS95 and MR94 Alpha Boo
c
	real teff,logg,mic

c
c original trend used for N6397 back in 2001
c	mic = 2.5 - 1.5*(logg-0.5)/4.0     

c
c New trend, based on Arcturus, 47Tuc star 6, and sun.  AMcW. 15/Aug./2003
c
c       mic = 1.0 + (4.5-logg)*(1.0/6.0)   !this was for run#3
c       mic = 2.2 + -logg*(4.0/15.0)       ! run #4 (add 0.3 km/s at logg=1.5)

        mic = 1.0 + (4.44-logg) * 0.6/(4.44-1.62)     ! linear between sun and Arcturus AMcW 7/2/2005


	return
	end








	subroutine read_kurucz_data(feh)

        common /akurucz/tkur1,tkur2,gkur1,gkur2,bckur1,bckur2
	real tkur1(450),gkur1(450),bckur1(450)
	real tkur2(450),gkur2(450),bckur2(450)

        common /ikurucz/nkur1,nkur2
        integer nkur1,nkur2

        real feh
	character*80 line

	if (feh.lt.-1.00 .or. feh.gt.-0.50) then
           write(6,'(a)')'can only get BC data for [Fe/H]=-0.5,-1.0 '
	   stop
	endif

	open(unit=7,file='ubvm05k2.datcd',status='old')
	read(7,'(a80)')line
	do i=1,450
           read(7,100,end=900)it,gkur1(i),bckur1(i)
	   tkur1(i) = real(it)
	enddo
 900	continue
	nkur1 = i - 1

	open(unit=8,file='ubvm10k2.datcd',status='old')
	read(8,'(a80)')line

	do i=1,450
           read(8,100,end=901)it,gkur2(i),bckur2(i)
	   tkur2(i) = real(it)
	enddo
 901	continue
	nkur2 = i - 1

	close(unit=7)
	close(unit=8)

 100    format(6x,i6,f6.2,56x,f8.3)
	return
	end




        subroutine compute_stellar_radii(l,teff,r)

	real l,teff,r

	tsun = 5777.
	r = sqrt(l)*(tsun/teff)**2

	return
	end
