      subroutine inmodel
c******************************************************************************
c     This subroutine reads in the model
c******************************************************************************

      implicit real*8 (a-h,o-z)
      include 'Chars.com'
      include 'Atmos.com'
      include 'Linex.com'
      include 'Mol.com'
      include 'Quants.com'
      include 'Factor.com'
      include 'Dummy.com'
      real*8 rhox(75), molweight(75) 
      real*8 element(95), logepsilon(95)
      real*8 kaprefmass(75)
      character list*80, list2*70


c*****Read in the key word to define the model type
      rewind filemodel
      read (filemodel,108) modtype
108   format (a10)
      call out (21)

c*****Read a comment line (usually describing the model)
      read (filemodel,106) moditle
106   format (a80)                                                       

c*****Read the number of depth points
      read (filemodel,208) list
      list2 = list(11:)
      read (list2,*) ntau

c*****EITHER: Read in a model from the output of the ATLAS code.  This
c     modtype is called "KURUCZ".  On each line the numbers are:
c     rhox, t, pgas, ne, and Rosseland mean opacity (cm^2/gm), and
c     two numbers not used by MOOG.  
      if (modtype .eq. 'KURUCZ    ') then
         do i=1,ntau
            read (filemodel,*) rhox(i),t(i),pgas(i),ne(i),kaprefmass(i)
         enddo
c     OR: Read in a model from the output of the MARCS code.  This modtype
c     type is called "BEGN".  On each line the numbers are:
c     tauross, t, log(pg), log(pe), mol weight, and kappaross.
      elseif (modtype .eq. 'BEGN      ') then
         do i=1,ntau
            read (filemodel,*) tauref(i),t(i),pgas(i),ne(i),
     .                          molweight(i),  kaprefmass(i)
         enddo
c     OR: Read in a model generated from ATLAS, but without accompanying
c     opacities.  MOOG will need to generate the opacities internally,
c     using a reference wavelength that it reads in before the model.
      elseif (modtype .eq. 'KURTYPE   ') then
         read (filemodel,*) wavref    
         do i=1,ntau
            read (filemodel,*) rhox(i),t(i),pgas(i),ne(i)
         enddo
c     OR: Read in a generic model that has a tau scale at a specific 
c     wavelength that is read in before the model.  
c     MOOG will need to generate the opacities internally.
      elseif (modtype .eq. 'GENERIC   ') then
         read (filemodel,*) wavref    
         do i=1,ntau
            read (filemodel,*) tauref(i),t(i),pgas(i),ne(i)
         enddo
c     OR: quit in utter confusion if those model types are not specified
      else
         write (*,*) 'permitted model types are:'
         write (*,*) '   KURUCZ, BEGN, KURTYPE, or GENERIC'
         write (*,*) 'MOOG quits!'
         stop
      endif

c*****Convert from logarithmic Pgas scales, if needed
      if (pgas(ntau)/pgas(1) .lt. 10.) then
         do i=1,ntau                                                    
            pgas(i) = 10.0**pgas(i)                                            
         enddo
      endif

c*****Convert from logarithmic Ne scales, if needed
      if(ne(ntau)/ne(1) .lt. 20.) then
         do i=1,ntau                                                    
            ne(i) = 10.0**ne(i)                                                
         enddo
      endif

c*****Convert from Pe to Ne, if needed
      if(ne(ntau) .lt. 1.0e7) then
         do i=1,ntau                                                    
            ne(i) = ne(i)/1.38054d-16/t(i)                                 
         enddo
      endif

c*****Read the microturbulence (either a single value to apply to 
c     all layers, or a value for each of the ntau layers). 
c     Conversion to cm/sec from km/sec is done if needed
      read (filemodel,101) (vturb(i),i=1,6)                                   
101   format (6e13.0)                                                     
      if (vturb(2) .ne. 0.) then
         read (filemodel,101) (vturb(i),i=7,ntau) 
      else
         do i=2,ntau                                                    
            vturb(i) = vturb(1)                                                
         enddo
      endif
      if (vturb(1) .lt. 100.) then
         write (moditle(57:64),157) vturb(1)
157      format ('vt=',f5.2)
         do i=1,ntau
            vturb(i) = 1.0e5*vturb(i)
         enddo
      else
         write (moditle(57:64),157) vturb(1)/1.0e5
      endif

c*****Copy abundance data from xabu0 to xabu
	do i = 1,95
           xabu(i) = xabu0(i)
	enddo
c*****Read in the abundance data, storing the original abundances in xabu
      read (filemodel,208) list
208   format(a80)
      list2 = list(11:)
      read (list2,*) natoms,abscale
      if(natoms .ne. 0) 
     .         read (filemodel,*) (element(i),logepsilon(i),i=1,natoms) 
      xhyd = 10.0**xabu(1)                                             
      xabund(1) = 1.0                                                    
      xabund(2) = 10.0**xabu(2)/xhyd                                   
      do i=3,95                                                      
         xabund(i) = 10.0**(xabu(i)+abscale)/xhyd                           
         xabu(i) = xabund(i)
      enddo
      if (natoms .ne. 0) then
         do i=1,natoms                                                  
            xabund(idint(element(i))) = 10.0**logepsilon(i)/xhyd
            xabu(idint(element(i))) = 10.0**logepsilon(i)/xhyd
         enddo
      endif

c*****Compute the mean molecular weight, ignoring molecule formation
c     in this approximation (maybe make more general some day?)
      wtnum = 0.
      wtden = 0.
      do i=1,95
         wtnum = wtnum + xabund(i)*xam(i)
         wtden = wtden + xabund(i)
      enddo
      wtmol = wtnum/(xam(1)*wtden)
      if (modtype .ne. 'BEGN      ') then
         do i=1,ntau
            molweight(i) = wtmol
         enddo
      endif

c*****Compute the density 
      do i=1,ntau                                                    
         rho(i) = pgas(i)*molweight(i)*1.6606d-24/(1.38054d-16*t(i))
      enddo

c*****Calculate the fictitious number density of hydrogen, and then
c     get the number densities of neutral hydrogen and helium by 
c     running through the molecular equilibrium routine with
c     constituents H I, H II, H2, He I, and He II.
c     Note:  ph = (-b1 + dsqrt(b1*b1 - 4.0*a1*c1))/(2.0*a1)
      iatom = 1
      dummy=partfn (dble(iatom),iatom)
      do i=1,ntau    
         th = 5040.0/t(i)         
         ah2 = 10.0**(-(12.7422+(-5.1137+(0.1145-0.0091*th)*th)*th))        
         a1 = (1.0+2.0*xabund(2))*ah2                                       
         b1 = 1.0 + xabund(2)                                               
         c1 = -pgas(i)         
         ph = (-b1/2.0/a1)+dsqrt((b1**2/(4.0*a1*a1))-(c1/a1))
         nhtot(i) = (ph+2.0*ph*ph*ah2)/(1.38054d-16*t(i))                   
      enddo
      amol(1) = 1.1
      amol(2) = 101.0
      amol(3) = 2.1
      nmolhold = nmol
      nmol = 3
      iprhold = ipr(2)
      if (ipr(1) .eq. 2) then
         ipr(2) = 2
         write (nwrite,1001)
1001     format (/'THE FOLLOWING EQUILIBRIUM COMPUTATIONS GET ',
     .           'H I AND HE I DENSITIES')
      else
         ipr(2) = 1
      endif
      call eqlib
      nmol = nmolhold
      ipr(2) = iprhold
      do i=1,ntau
         nhneutral(i) = xamol(1,i)
         nheneutral(i) = xamol(2,i)
      enddo

c*****Read in the names of molecules to be used in molecular equilibrium
      if (ipr(2) .ne. 0) then
         read (filemodel,208) list
         list2 = list(11:)
         read (list2,*) nmol
         read (filemodel,*) (amol(i),i=1,nmol)                                 
      endif

c*****SPECIAL NEEDS: for KURUCZ models, to create the optical depth array,
c     and to convert kaprefs to our units
      if (modtype .eq. 'KURUCZ    ') then
         first = rhox(1)*kaprefmass(1)
         tottau = rinteg(rhox,kaprefmass,tauref,ntau,first) 
         tauref(1) = first
         do i=2,ntau
            tauref(i) = tauref(i-1) + tauref(i)
         enddo
         do i=1,ntau
            kapref(i) = kaprefmass(i)*rho(i)
         enddo
c     SPECIAL NEEDS: for BEGN models, to convert kaprefs to our units
      elseif (modtype .eq. 'BEGN      ') then
         do i=1,ntau                                                    
            kapref(i) = kaprefmass(i)*rho(i)
         enddo
c     SPECIAL NEEDS: for KURTYPE models, to create internal kaprefs,
c     and to compute taurefs from the kaprefs converted to mass units
      elseif (modtype .eq. 'KURTYPE   ') then
         call opacit (1,wavref)
         do i=1,ntau                                                    
            kaprefmass(i) = kapref(i)/rho(i)
         enddo
         first = rhox(1)*kaprefmass(1)
         tottau = rinteg(rhox,kaprefmass,tauref,ntau,first) 
         tauref(1) = first
         do i=2,ntau
            tauref(i) = tauref(i-1) + tauref(i)
         enddo
c     SPECIAL NEEDS: for generic models, to create internal kaprefs,
      elseif (modtype .eq. 'GENERIC   ') then
         call opacit (1,wavref)
      endif

c*****Convert from logarithmic optical depth scales, or vice versa.
c     xref will contain the log of the tauref
      if(tauref(1) .lt. 0.) then
         do i=1,ntau                                                    
            xref(i) = tauref(i)
            tauref(i) = 10.0**xref(i)                                        
         enddo
      else
         do i=1,ntau                                                    
            xref(i) = dlog10(tauref(i))                                        
         enddo
      endif

c*****Locate the atmosphere level where tauref is near 0.5
      do i=1,ntau
         if (tauref(i) .ge. 0.5) go to 180
      enddo
180   jtau5 = i

c*****Write information to output
      call out (1)
      call out (8)

      return

      end                                                                

