
      subroutine smooth (line)
c******************************************************************************
c     This subroutine prepares synthesis data for plotting by smoothing
c     the data in various ways.
c******************************************************************************
 
      implicit real*8 (a-h,o-z)
      include 'Chars.com'
      include 'Atmos.com'
      include 'Linex.com'
      include 'Pstuff.com'
      include 'Equivs.com'
      logical gaussflag, rotateflag, lorenflag, macroflag


c*****initialize parameters
      nsyn = 1
      p0 = 1.
      gaussflag = .false.
      rotateflag = .false.
      lorenflag = .false.
      macroflag = .false.


c*****on entering, rewind the unsmoothed and smoothed spectrum output 
c     files, and get the synthesis range parameters from the 'dump' file
      rewind ndump
      rewind n2dump
55    read (ndump,1002) moditle
      if (moditle(1:15).eq.'        element' .or.  
     .    moditle(1:15).eq.' ALL abundances' .or.
     .    moditle(1:15).eq.'Isotope Ratio: ') go to 55
1002  format (a80)
      read (ndump,*) start, stop, step
      kount = int((stop - start + (step/4.0) )/step) + 1
      rewind ndump


c*****branch to the desired smoothing function
      if     (smtype .eq. 'n') then
         go to 40 
      elseif (smtype .eq. 'l') then
         lorenflag = .true.
      elseif (smtype .eq. 'g') then
         gaussflag = .true.
      elseif (smtype .eq. 'v') then
         rotateflag = .true.
      elseif (smtype .eq. 'c') then
         rotateflag = .true.
         gaussflag = .true.
      elseif (smtype .eq. 'm') then
         macroflag = .true.
      elseif (smtype .eq. 'd') then
         macroflag = .true.
         gaussflag = .true.
      elseif (smtype .eq. 'r') then
         macroflag = .true.
         rotateflag = .true.
         gaussflag = .true.
      endif


c*****compute a stellar rotational broadening function
c     this follows D. F. Gray, 1976, "The Obs. & Anal. of Stell. Phot", p394-9
30    if (rotateflag .eqv. .false.) go to 50
32    array = 'GIVE THE STELLAR vsini [0.0]: '
      nchars = 30
      if (line .gt. 0) then
         call getasci (nchars,line)
         call number (nchars,line,vsini)
         if (vsini .eq. -9999.) vsini = 0.
      endif
      if (vsini .lt. 0.0) go to 32
31    array = 'GIVE THE LIMB DARKENING COEFFICIENT [0.0]: '
      nchars = 43
      if (line .gt. 0) then
         call getasci (nchars,line)
         call number (nchars,line,epsilon)
         if (epsilon .eq. -9999.) epsilon = 0.
      endif
      if (epsilon .lt. 0.0) go to 31
      dlamlim = (start+stop)/2.*vsini/3.0e5
      if (step .ge. dlamlim) then
         rotateflag = .false.
         go to 50
      endif
      pi = 3.141527
      bottom = dlamlim*pi*(1.-epsilon/3.)
      c1 = 2.*(1.-epsilon)/bottom
      c2 = 0.5*epsilon*pi/bottom
      prot0 = c1 + c2
      powerrot = prot0
      jdelrot = dint(dlamlim/step)
      if (jdelrot .gt. 1000) then
         write (*,1026) 
1026     format ('ERROR: ROTATIONAL PROFILE TOO BIG!'/  
     .           '       INCREASE SPECTRUM STEP SIZE?')
         smtype = 'e'
         return
      elseif (jdelrot .gt. kount/4) then
         write (*,1028)
1028     format ('ERROR: ROTATIONAL PROFILE COVERS WHOLE ',
     .           'SPECTRUM!'/
     .           '       INCREASE SYNTHESIS LENGTH OR DECREASE ',
     .           'STEP SIZE?')
         smtype = 'e'
         return
      endif
      do i=1,jdelrot
         dlam = (i-1)*step
         term = 1. - (dlam/dlamlim)**2
         prot(i) = c1*dsqrt(term) + c2*term
         powerrot = powerrot + 2.0*prot(i)
      enddo
         
c*****compute a macroturbulent smoothing function (uses subroutine vmacro)
50    if (macroflag .eqv. .false.) go to 10
51    array = 'GIVE THE MACROTURBULENT VELOCITY [0.0]: '
      nchars = 39
      if (line .gt. 0) then
         call getasci (nchars,line)
         call number (nchars,line,velmac)
         if (velmac .eq. -9999.) velmac = 0.
      endif
      if (velmac .lt. 0.0) go to 51
      if (velmac .eq. 0.) then
         macroflag = .false.
         go to 10
      endif
      wavemac = (start+stop)/2.*velmac/3.0e5
      powermac = 1.0
      do i=1,1000
         wavei = step*i/wavemac
         pmac(i) = vmacro(wavei)
         powermac = powermac + 2.0 *pmac(i)
         if (pmac(i) .lt. 0.002) then
            jdelmac = i
            go to 15
         endif
      enddo
15    if (jdelmac .gt. 1000) then
         write (*,1025) wavemac
1025     format ('ERROR: MACROTURBULENT PROFILE TOO BIG! ',
     .           '(WAVELENGTH WIDTH=',f6.2,')'/
     .           '       INCREASE SPECTRUM STEP SIZE?')
         smtype = 'e'
         return
      elseif (jdelmac .gt. kount/4) then
         write (*,1022)
1022     format ('ERROR: MACROTURBULENT PROFILE COVERS WHOLE ',
     .           'SPECTRUM!'/
     .           '       INCREASE SYNTHESIS LENGTH OR DECREASE ',
     .           'STEP SIZE?')
         smtype = 'e'
         return
      endif   


c*****compute a Gaussian smoothing function
10    if (gaussflag .eqv. .false.) go to 20
11    array = 'GIVE THE FWHM OF THE GAUSSIAN FUNCTION [0.0]: ' 
      nchars = 46
      if (line .gt. 0) then
         call getasci (nchars,line)
         call number (nchars,line,half)
         if (half .eq. -9999.) half = 0.
      endif
      if (half .lt. 0.0) go to 11
      if (half .eq. 0.) then
         gaussflag = .false.
         go to 20
      endif
      sigma = half/2.
      aa = 0.6932/sigma**2
      power = 1.0
      do i=1,1000
         p(i) = dexp(-aa*(step*i)**2 )
         power = power + 2*p(i)
         if (p(i) .lt. 0.02) then
            jdel = i
            go to 25 
         endif
      enddo
25    if (jdel .gt. 1000) then
         write (*,1029) sigma
1029     format ('ERROR: GAUSSIAN PROFILE TOO BIG! ',
     .           '(HALF WIDTH=',f6.2,')'/
     .           '       INCREASE SPECTRUM STEP SIZE?')
         smtype = 'e'
         return   
      elseif (jdel .gt. kount/4) then
         write (*,1021)
1021     format ('ERROR: GAUSSIAN PROFILE COVERS WHOLE ',
     .           'SPECTRUM!'/
     .           '       INCREASE SYNTHESIS LENGTH OR DECREASE ',
     .           'STEP SIZE?')
         smtype = 'e'
         return
      endif


c*****compute a Lorenzian smoothing function
20    if (lorenflag .eqv. .false.) go to 40
21    array = 'GIVE THE FWHM OF THE LORENTZIAN FUNCTION [0.0]: '
      nchars = 48
      if (line .gt. 0) then
         call getasci (nchars,line)
         call number (nchars,line,half)
         if (sigma .eq. -9999.) half = 0.
      endif
      if (half .lt. 0.0) go to 21
      if (half .eq. 0.) then
         lorenflag = .false.
         go to 40
      endif
      sigma = half/2.
      power = 1.0
      do i=1,1000
         p(i) = ((sigma**2)/((sigma**2)+((step*i)**2)))
         power = power + 2.0 *p(i)
         if (p(i) .lt. 0.02) then
            jdel = i
            go to 35 
         endif
      enddo    
35    if (jdel .gt. 1000) then
         write (*,1030) sigma
1030     format ('ERROR: LORENZIAN PROFILE TOO BIG! ',
     .           '(HALF WIDTH=',f6.2,')'/
     .           '       INCREASE SPECTRUM STEP SIZE?')
         smtype = 'e'
         return  
      elseif (jdel .gt. kount/4) then
         write (*,1031)
1031     format ('ERROR: LORENZIAN PROFILE COVERS WHOLE ',
     .           'SPECTRUM!'/
     .           '       INCREASE SYNTHESIS LENGTH OR DECREASE ',
     .           'STEP SIZE?')
         smtype = 'e'
         return
      endif

      istat = ivcleof (line,1)
      istat = ivmove (line-1,1)
      write (*,1024) sigma,(p(i),i=1,500)
 1024 format('ERROR: LORENZ PROFILE TOO BIG! (HALF WIDTH=',
     .      f6.2,')        THE PROFILE;'/(15f5.2))
      smtype = 'e'
      return


c*****now read in the raw spectrum and flip to a depth scale
40    noff = 80*(nsyn-1)
      abitle(noff+1:noff+12) = '[M/H] 0.00  '
      nabunds = 0
41    read (ndump,1002,end=2000) array
      if     (array(1:15).eq.' ALL abundances') then
         abitle(noff+6:noff+10) = array(56:60)
         go to 41
      elseif (array(1:15).eq.'        element') then
         nabunds = nabunds + 1
         if (nabunds .le. 7) then
            ioff = noff + 12 + 9*(nabunds-1)
            abitle(ioff+1:ioff+2) = array(17:18)
            abitle(ioff+3:ioff+7) = array(34:38)
            abitle(ioff+8:ioff+9) = '  '
         endif
         go to 41
      elseif (array(1:15).eq.'Isotope Ratio: ') then
         nabunds = nabunds + 1
         if (nabunds .le. 7) then
            ioff = noff + 12 + 9*(nabunds-1)
            abitle(ioff+1:ioff+4) = array(27:30)
            abitle(ioff+5:ioff+5) = ' '
            do k=33,44
               if (array(k:k) .ne. ' ') then
                  abitle(ioff+6:ioff+8) = array(k:k+2)
                  go to 60
               endif
            enddo
60          abitle(ioff+9:ioff+9) = ' '
         endif
         go to 41
      endif
      read (ndump,1002,end=2000)
      read (ndump,1003,end=2000) (y(i),i=1,kount)
1003  format (10f7.4)
      do i=1,kount
         y(i) = 1.0 - y(i)
      enddo
      if (iunits .eq. 2) then
         do i=1,kount/2
            temp = y(i)
            y(i) = y(kount+1-i)
            y(kount+1-i) = temp
         enddo
      endif


c*****apply the rotational broadening if desired
      if (rotateflag .eqv. .true.) then
         min = jdelrot + 1
         max = kount - jdelrot
         do i=1,jdelrot
            z(i) = 1.
            z(kount-i+1) = 1.
         enddo
         do i=min,max
            z(i) = prot0*y(i)
            do j=1,jdelrot
               z(i) = z(i) + prot(j)*(y(i-j) + y(i+j))
            enddo
            z(i) = z(i)/powerrot
         enddo
         do i=1,kount
            y(i) = z(i)
         enddo
      endif


c*****apply the macroturbulent broadening if desired
      if (macroflag .eqv. .true.) then
         min = jdelmac + 1
         max = kount - jdelmac
         do i=1,jdelmac
            z(i) = 1.
            z(kount-i+1) = 1.
         enddo
         do i=min,max
            z(i) = p0*y(i)
            do j=1,jdelmac
               z(i) = z(i) + pmac(j)*(y(i-j) + y(i+j))
            enddo
            z(i) = z(i)/powermac
         enddo
         do i=1,kount
            y(i) = z(i)
         enddo
      endif


c*****apply the Gaussian or Lorenzian smoothing if desired (this
c     is an either/or situation; only one of these can apply.
      if (gaussflag .eqv. .true. .or. lorenflag .eqv. .true.) then
         min = jdel + 1
         max = kount - jdel
         do i=1,jdel
            z(i) = 1.
            z(kount-i+1) = 1.
         enddo
         do i=min,max
            z(i) = p0*y(i)
            do j=1,jdel
               z(i) = z(i) + p(j)*(y(i-j) + y(i+j))
            enddo
            z(i) = z(i)/power
         enddo
         do i=1,kount
            y(i) = z(i)
         enddo
      endif


c*****move the smoothed spectrum (or unsmoothed, if nothing has
c     been done to the y-array) into the appropriate array
      do i = 1,kount
         chunk(i,nsyn) = y(i)
      enddo


c*****compute the wavelength array; must be done for each synthetic
c     spectrum because of the way the equivalences were set up
      do i=1,kount
         xsyn(i) = start + (i-1)*step
      end do
 

c*****dump the smoothed spectrum in a MONGO-style set of 
c     (wavelength,flux) point pairs
      write (n2dump,1005) kount,start,stop,step
1005  format ('the number of points per synthesis = ',i5/
     .       'start = ',f10.3,5x,'stop = ',f10.3,5x,'step = ',f10.3)
      if (xsyn(1) .le. 100.0) then
         write (n2dump,1009) (xsyn(i),chunk(i,nsyn),i=1,kount)
      else 
         write (n2dump,1008) (xsyn(i),chunk(i,nsyn),i=1,kount)
      endif
1009  format (f10.6,'  ',f10.5)
1008  format (f10.3,'  ',f10.5)
      nsyn = nsyn + 1
      go to 40


c*****exit the routine normally
2000  nsyn = nsyn - 1
      return



      end



