	program eab1
c
c a routine to compute the atmosphere parameters of the
c cmd sub-groups for NGC 6397, from observed photometry
c
        real v(30),bv(30),teff(30),logg(30),mic(30),frac(30),r(30)
	real feh
	integer nbox,nstar(30)
	data logg/30*2.5/

c
c here are the parameters specific to NGC 6397
c
	feh  =  -2.0
c       dist =  11.71  oops I used the wrong distance at first
c this distance and reddening from Gratton et al. 2003, A&A 408, 529
	dist =  12.58    
	ebv  =   0.183

	call read_kurucz_data(feh)
	call read_gc_boxes(v,bv,frac,nstar,nbox)
	call de_redden(v,bv,ebv,nbox)
	call compute_params(v,bv,feh,teff,logg,mic,r,dist,nbox)
	call write_params(v,bv,teff,logg,mic,r,frac,nstar,nbox)

	stop
	end


	subroutine read_gc_boxes(v,bv,frac,nstar,nbox)
c
c a simple routine to read the cmd information
c
	character*40 line
	real v(30),bv(30),frac(30)
	integer nbox,nstar(30)

	open(unit=8,file='fractions.dat',status='old')
	
	read(8,'(a40)')line
	read(8,'(a40)')line

	do i =1,30
	   read(8,*,end=900)v(i),bv(i),frac(i),nstar(i)
	enddo
 900	continue
	nbox = i-1

	return
	end



	subroutine de_redden(v,bv,ebv,nbox)

	real v(30),bv(30),ebv,av
c
c correct B-V for reddning and V for extinction using the
c reddening law of Winkler, 1997, MNRAS, 287, 481.
c
	av = 3.136 * ebv
	do i = 1,nbox
	   bv(i) = bv(i) - ebv
	   v(i) = v(i) - av
	enddo

	return
	end




        subroutine write_params(v,bv,teff,logg,mic,r,frac,nstar,nbox)

	real v(30),bv(30),teff(30),logg(30),mic(30),r(30),frac(30)
	integer nstar(30)

	write(6,100)
 100	format('    Vavg   (B-V)  Teff   logg   micro  radius',
     %         '   fraction  Nstar',/)
        write(6,200)(v(i),bv(i),teff(i),logg(i),mic(i),r(i),frac(i),
     %               nstar(i),i=1,nbox)
 200	format(2f8.3,f7.0,f7.3,f7.2,f8.2,f10.5,i8)

	return
	end




	subroutine compute_params(v,bv,feh,teff,logg,mic,r,dist,nbox)

	real v(30),bv(30),teff(30),logg(30),mic(30),r(30)
	real l(30),mass,dist,mv
	integer nbox

c
c note that I assume a turnoff mass of 0.8Msun
c
	mass = 0.80

	do i = 1,nbox
	   mv = v(i) - dist
	   logg(i) = 2.5
	   call get_teff(bv(i),mv,feh,logg(i),teff(i))
	   call get_luminosity(v(i),bv(i),teff(i),logg(i),feh,dist,l(i))
	   call compute_logg(l(i),mass,teff(i),logg(i))
c
c do it twice to converge on BC, logg, and BHB Teff
c
	   call get_teff(bv(i),mv,feh,logg(i),teff(i))
	   call get_luminosity(v(i),bv(i),teff(i),logg(i),feh,dist,l(i))
	   call compute_logg(l(i),mass,teff(i),logg(i))
           call compute_stellar_radii(l(i),teff(i),r(i))

	   call compute_micro(teff(i),logg(i),mic(i))
	enddo

	return
	end





	subroutine get_teff(bv,mv,feh,logg,teff)

	real bv,mv,feh,logg,teff
c
c Assume (B-V) colors, with calibration from Alonso (1999) for
c giants up to B-V=0.35, and Alonso (1996) for dwarf stars.
c


c giants
        th1 = 0.5716 + 0.5404*bv - 6.126e-2 * bv**2 - 4.862e-2*feh*bv
     %        -1.777e-2 * feh - 7.969e-3 * feh**2
        th2 = 0.6177 + 0.4354*bv -4.025e-3*bv**2 + 5.204e-2*bv*feh
     %        -0.1127*feh - 1.385e-2*feh**2


	if (bv.lt.0.70) then
	   teffrgb = 5040./th1
	elseif (bv.gt.0.80) then
	   teffrgb = 5040./th2
	else
	   f = (bv-0.70)/0.10
	   teff1 = 5040./th1
	   teff2 = 5040./th2
	   teffrgb = teff1 + f*(teff2-teff1)
	endif

c dwarfs

	theta = 0.541 + 0.533*bv + 0.007*bv**2 - 0.019*bv*feh
     %          -0.047*feh - 0.011*feh**2
	teffdwf = 5040./theta

c subgiants
c
c NOTE: the mv cutoffs are based on the NGC 6397 cmd; logg may be better?
c

	if (mv.gt.3.75) then
	   teff=teffdwf
	elseif (mv.lt.3.13) then
	   teff = teffrgb
	elseif (mv.le.3.75 .and. mv.ge.3.13) then
	   f = (mv-3.13)/0.62
	   teff = teffrgb + f*(teffdwf-teffrgb)
	endif

	if (bv.lt.0.35) then
           call get_kurucz_teff(bv,logg,teff)
	   return
	endif

	return
	end




	subroutine get_luminosity(v,bv,teff,logg,feh,dist,l)

	real v,bv,teff,dist,l,mv,mbol_sun,logg,bc,mbol

	mbol_sun =  4.74
	mbol = v - dist + bc(teff,logg,feh)

	l = 10.0**(-0.4*(mbol-mbol_sun))

	return
	end



	real function bc(teff,logg,feh)

	real logg,teff,teff_lo,teff_hi
	real bc_entry,feh
        real bc_lo,bc_hi

	teff_lo = aint(teff/250.0) * 250.0
	teff_hi = aint(teff/250.0) * 250.0 + 250.0

        bc_lo = bc_entry(teff_lo,logg,feh)
        bc_hi = bc_entry(teff_hi,logg,feh)

        bc = bc_lo + (teff-teff_lo)*(bc_hi-bc_lo)/250.0

	return
	end






	real function bc_entry(teff,logg,feh)
c
c here use the Kurucz tables of computed BC
c
        common /akurucz/tkur,gkur,bckur,bvkur
	real tkur(450),gkur(450),bckur(450),bvkur(450)

        common /ikurucz/nkur
        integer nkur

	real logg,teff,feh

	bc_entry = -99.99
c
c get value from -2 [Fe/H] table
c

	do i = 1,nkur
	   if (teff-tkur(i) .le. 50.) then

	      if (logg.lt.gkur(i)) then    ! if lower than first logg
	         bc_entry = bckur(i)
	         return
	      endif
	     
              do j = 1,10
	         if (tkur(i+j).eq.tkur(i)) then
	            if (gkur(i+j-1).le.logg .and.
     %                              gkur(i+j).ge.logg) then
                       f = (logg-gkur(i+j-1))/(gkur(i+j)-gkur(i+j-1))
                       bc_entry = bckur(i+j-1) + 
     %                           f*(bckur(i+j)-bckur(i+j-1))
                       return
                    endif
                 else
                    bc_entry = bckur(i+j-1)    ! if higher than last logg
                    return
                 endif
              enddo

           endif
        enddo

	write(6,'(a)')'Error: BC not found'
	stop
	end



c	real function bc(teff,logg,feh)
c
c here use the Kurucz tables of computed UBV colors and BC
c
c at the moment I only have data for [Fe/H]=-2
c
c       common /akurucz/tkur,gkur,bckur,bvkur
creal tkur(450),gkur(450),bckur(450),bvkur(450)

c       common /ikurucz/nkur
c       integer nkur

creal logg

cbc = -99.99

cdo i = 1,nkur
c   if (teff-tkur(i) .le. 250.) then
c      if (logg.lt.gkur(i)) then
c         bc = bckur(i)
c         return
c      endif
c     
c             do j = 1,10
c         if (tkur(i+j).eq.tkur(i)) then
c            if (gkur(i+j-1).le.logg .and.
c    %                              gkur(i+j).ge.logg) then
c                      f = (logg-gkur(i+j-1))/(gkur(i+j)-gkur(i+j-1))
c                      bc = bckur(i+j-1) + f*(bckur(i+j)-bckur(i+j-1))
c                      return
c                   endif
c                else
c                   bc = bckur(i+j-1)
c                   return
c                endif
c             enddo
c          endif
c       enddo


cwrite(6,'(a)')'Error: BC not found'
cstop
cend




        subroutine compute_logg(l,mass,teff,logg)

	real l,mass,teff,logg,logg_sun

	tsun = 5777.
	logg_sun = 4.43775
        logg = logg_sun+alog10(mass) - alog10(l) + 4.*alog10(teff/tsun)

	return
	end



	subroutine compute_micro(teff,logg,mic)
c
c a very rough estimation of the trend of microturbulence with logg
c
c FEEL FREE TO IMPROVE THIS ROUTINE
c
c made-up, with some input from MPSS95 and MR94 Alpha Boo
c
	real teff,logg,mic

	mic = 2.5 - 1.5*(logg-0.5)/4.0

	return
	end






        subroutine get_kurucz_teff(bv,logg,teff)

        common /akurucz/tkur,gkur,bckur,bvkur
	real tkur(450),gkur(450),bckur(450),bvkur(450)

        common /ikurucz/nkur
        integer nkur

	real bv,feh,logg,teff

c get the b-v for the first model 

	do i = 1,nkur-1
           if (gkur(i).le.logg .and. gkur(i+1).gt.logg) then
              f   = (logg-gkur(i))/(gkur(i+1)-gkur(i))
              bv1 = bvkur(i) + f*(bvkur(i+1)-bvkur(i))
              teff1 = tkur(i)
              goto 900
           endif
	enddo

900	continue

c now loop though until bv is bracketed; then compute teff

	do j = i+1,nkur-1
           if (gkur(j).le.logg .and. gkur(j+1).gt.logg) then
              f   = (logg-gkur(j))/(gkur(j+1)-gkur(j))
              bv2 = bvkur(j) + f*(bvkur(j+1)-bvkur(j))
              teff2 = tkur(j)

              if (bv1.ge.bv .and. bv2.le.bv) then
                 f = (bv-bv2)/(bv1-bv2)
                 teff = teff2 + f*(teff1-teff2)
                 return
              else
                 bv1 = bv2
                 teff1 = teff2
              endif
	   endif
	enddo

	write(6,'(a)')'Error: B-V out of Kurucz table bounds'
	stop
	end





	subroutine read_kurucz_data(feh)

        common /akurucz/tkur,gkur,bckur,bvkur
	real tkur(450),gkur(450),bckur(450),bvkur(450)

        common /ikurucz/nkur
        integer nkur

        real feh
	character*80 line

	if (feh.ne.-2.0) then
           write(6,'(a)')'can only get BC data for [Fe/H]=-2 at present'
	   stop
	endif

	open(unit=7,file='ubvm20k2.datcd',status='old')
	read(7,'(a80)')line

	do i=1,450
           read(7,100,end=900)it,gkur(i),bckur(i),bvkur(i)
	   tkur(i) = real(it)
	enddo
 900	continue
	nkur = i - 1
	close(unit=7)

 100    format(6x,i6,f6.2,56x,f8.3,8x,f8.3)
	return
	end




        subroutine compute_stellar_radii(l,teff,r)

	real l,teff,r

	tsun = 5777.
	r = sqrt(l)*(tsun/teff)**2

	return
	end
