	program eab2
	implicit real*8 (a-h,o-z)
c
c a program which will compute the integrated-light EW of a single line,
c or in future will compute the integrated-light spectrum for a specified
c wavelength interval
c
	real*8 wave,ewavg,wtsum,ewsum,tol,abund
	character*2 mode

c tolerance for EW match

	tol = 0.02

c set start abundance (later take this from Moog)

        abund = 6.50


	call read_gc_parameters (mode)

        call read_line_info(iatom,wave,ewobs)


        if (mode.eq.'EW') then 

c loop until EW convergence

           do i = 1,50

c compute EW for each of the 25 cmd sub-regions

              do nmodel = 1,25
                 call produce_model_atmosphere(nmodel,iatom,abund) 
                 call system('ewfind')                !moog EW system call 
                 call organize_output(nmodel)
	      enddo

              call get_ews_and_fluxes
              call compute_ew_weights
              call compute_weighted_EW(ewavg)
c DEBUG
	write(6,'(a)')'iatom,abund,ewavg,ewobs'
	write(6,*)iatom,abund,ewavg,ewobs
c
              call estimate_next_abund(wave,ewavg,ewobs,abund)

              if (abs((ewavg-ewobs)/ewobs) .le. tol) then
                 call output_results(ewavg,iatom,abund)
	         stop
	      endif

	   enddo
	endif
c
c
c
c       elseif (mode.eq.'SS') then
c
c compute SS for each of the 25 cmd sub-regions
c
c          do nmodel = 1,25
c             call produce_model_atmosphere(nmodel,iatom,abund) 
c             call system('cmoog')                !moog SS system call 
c             call organize_output(i)
c          enddo
c          extract flux weight information from moog output files
c          extract predicted spectrum from moog output files
c          compute weights for each model 
c          compute weighted spectrum
c          write weighted spectrum
c
c       endif
c
c
	stop
	end




	subroutine read_gc_parameters(mode)
	implicit real*8 (a-h,o-z)

	common/rpars/v,bv,teff,glog,mic,feh,r,frac,sigv,sw
        real*8 v(30),bv(30),teff(30),glog(30),mic(30),r(30),frac(30)
        real*8 sigv,sw,feh

        common/ipars/nstar
	integer nstar(30)

	character*80 line,mode*2

	read(5,'(a80)')line
        read(5,'(a2)')mode

c 
c just set feh to -2.0 here, for NGC 6397
c
        feh = -2.0
c
c now read the globular cluster parameter information
c
	do i = 1,25
           read(5,*)v(i),bv(i),teff(i),glog(i),mic(i),r(i),frac(i),
     %              nstar(i)

c DEBUG
c       mic(i) = mic(i) -0.5
c       teff(i) = teff(i) + 200.0
c       glog(i) = glog(i) + 0.6
c
	enddo
c DEBUG
c       nstar(25) = 4*nstar(25)
        nstar(1) = 0
        nstar(2) = 0
        nstar(3) = 0
        nstar(4) = 0
        nstar(5) = 0
        nstar(6) = 0
        nstar(7) = 0
        nstar(8) = 0
        nstar(9) = 0

	return
	end




        subroutine read_line_info(iatom,wave,ewobs)
	implicit real*8 (a-h,o-z)

	real*8 wave,ewobs,atom
	integer iatom
	character*80 line

	open(unit=7,file='INPUT',status='old')

	read(7,'(a80)')line
	read(7,'(a80)')line
	read(line(16:25),*)atom
	iatom = nint(atom)
	read(line(6:15),*)wave
	read(line(76:80),*)ewobs

	close(unit=7)

	return
	end




        subroutine organize_output(imodel)
	implicit real*8 (a-h,o-z)
c 
c this routine will move the MOOG output files 
c
	integer imodel
	character*30 cmd1,cmd2

	cmd1 = 'cp OUTPUT output.'
	cmd2 = 'cp logfile log.'
	if (imodel.le.9) then
	    write(cmd1(18:18),'(i1)')imodel
	    write(cmd2(16:16),'(i1)')imodel
	elseif (imodel.ge.10) then
	    write(cmd1(18:19),'(i2)')imodel
	    write(cmd2(16:17),'(i2)')imodel
	endif
	call system(cmd1)
	call system(cmd2)
	return
	end





        subroutine compute_weighted_EW(ewavg)
	implicit real*8 (a-h,o-z)

        common/rline/ew,cflux,wt
        real*8 ew(30),cflux(30),wt(30)

	real*8 ewavg,wtsum,ewsum

	ewsum = 0.0
	wtsum = 0.0

	do i = 1,25
	   ewsum = ewsum + ew(i)*wt(i)
           wtsum = wtsum + wt(i)
	enddo
c DEBUG
        write(6,'(a)')'i,   EW(i),  frac_EW'
	do i = 1,25
	   few = ew(i)*wt(i)/wtsum
	   write(6,*)i,ew(i),few
	enddo
	ewavg = ewsum/wtsum
	return
	end




        subroutine estimate_next_abund(wave,ewcalc,ewobs,abund)
	implicit real*8 (a-h,o-z)

	real*8 wave,ewavg,ewobs,abund,ratio,rwlcomp,rwlgobs

c
c based on Moog's convergence code for Abfind
c
        ratio = ewobs/ewcalc
        rwlcomp = dlog10(ewcalc/wave) - 3.0
        rwlgobs = dlog10(ewobs/wave) - 3.0
        if (rwlcomp.lt.-5.2 .and. rwlgobs.lt.-5.2) then
           ratio = ratio
        elseif (rwlcomp.ge.-5.2 .and. rwlgobs.ge.-5.2) then
           ratio = ratio**2.0
        else
           ratio = ratio**1.5
        endif
	abund = abund+dlog10(ratio)

	return
	end





        subroutine produce_model_atmosphere(nmodel,iatom,abund) 
	implicit real*8 (a-h,o-z)

	common/rpars/v,bv,teff,glog,mic,feh,r,frac,sigv,sw
        real*8 v(30),bv(30),teff(30),glog(30),mic(30),r(30),frac(30)
        real*8 sigv,sw,feh

	real*8 abund
	integer iatom,nmodel

	i = nmodel

        call make_kurucz(teff(i),glog(i),feh,mic(i),iatom,abund)

	return
	end






        subroutine get_ews_and_fluxes
	implicit real*8 (a-h,o-z)

        common/rline/ew,cflux,wt
        real*8 ew(30),cflux(30),wt(30)

	character*80 line,fname*30

        fname = 'output.1'

	do i = 1,25
           if (i.lt.10) then
              write(fname(8:8),'(i1)')i
	   else
              write(fname(8:9),'(i2)')i
	   endif
	   open(unit=7,file=fname,status='old')
c
c read the 5 header lines and the line containing EW and continuum flux
c
	   do j = 1,6
	      read(7,'(a80)')line
	   enddo

c read ew and flux

           read(line(53:75),*)ew(i),cflux(i)

	   close(unit=7)
	enddo
	return
	end





        subroutine compute_ew_weights
	implicit real*8 (a-h,o-z)

	common/rpars/v,bv,teff,glog,mic,feh,r,frac,sigv,sw
        real*8 v(30),bv(30),teff(30),glog(30),mic(30),r(30),frac(30)
        real*8 sigv,sw,feh

        common/ipars/nstar
	integer nstar(30)

        common/rline/ew,cflux,wt
        real*8 ew(30),cflux(30),wt(30)

        wtsum = 0.0
	do i = 1,25
	   wt(i) = cflux(i) * r(i)**2 * dble(nstar(i))
c DEBUG
c          wt(i) = frac(i)
           wtsum = wtsum + wt(i)
c

	enddo
c DEBUG
c
	do i = 1,25
	   wt(i) = wt(i)/wtsum
	enddo
c

	
	return
	end




        subroutine output_results(ewavg,iatom,abund)
	implicit real*8 (a-h,o-z)
c DEBUG
        common/rline/ew,cflux,wt
        real*8 ew(30),cflux(30),wt(30)
c


	real*8 ewavg,abund
	integer iatom

        write(6,'(a)')'  EW_avg   IATOM  Abundance'
        write(6,'(f10.2,i8,f10.3)')ewavg,iatom,abund

c DEBUG
        write(6,'(a)')'  I      WT(I)'
	do i = 1,25
	   write(6,'(i5,2x,f10.6)')i,wt(i)
	enddo
	return
	end
